//--------------------------------------------------------------------------
// File and Version Information:
//
// Description:
//	Class template.
//
// Environment:
//
// Author List:
//
// Copyright Information:
//
//------------------------------------------------------------------------

//------------------------
// String Header First  --
//------------------------
#include <string>

//-----------------------
// This Class's Header --
//-----------------------
#include "HepgHist/HepgHist.hh"
//-------------
// C Headers --
//-------------
#include <assert.h>
#include <time.h>
//---------------
// C++ Headers --
//---------------
#include <iostream.h>

//-------------------------------
// Collaborating Class Headers --
//-------------------------------
#include "Edm/ConstHandle.hh"
#include "SimulationObjects/HEPG_StorableBank.hh"

#include "AbsEnv/AbsEnv.hh"

//-----------------------------------------------------------------------
// Local Macros, Typedefs, Structures, Unions and Forward Declarations --
//-----------------------------------------------------------------------

static const char rcsid[] = "$Id: $";

//----------------
// Constructors --
//----------------

HepgHist::HepgHist( 
		   const char* const theName, 
		   const char* const theDescription ) :
  AppModule( theName, theDescription )
{
}


//--------------
// Destructor --
//--------------

HepgHist::~HepgHist()
{
}

//--------------
// Operations --
//--------------

AppResult HepgHist::beginJob( AbsEvent* aJob )
{
  char* chform_var="idhep:I*4,"
    "E:R*4,Px:R*4,Py:R*4,Pz:R*4,"
    "Vx:R*4,Vy:R*4,Vz:R*4";
  vector<void*> var_addr;
  var_addr.push_back(&_var.idhep);
  var_addr.push_back(&_var.E);
  var_addr.push_back(&_var.Px);
  var_addr.push_back(&_var.Py);
  var_addr.push_back(&_var.Pz);
  var_addr.push_back(&_var.Vx);
  var_addr.push_back(&_var.Vy);
  var_addr.push_back(&_var.Vz);

  _manager = new HepRootFileManager("HepgHist.root",
                                    HepFileManager::HEP_REMOVE);
  _ntuple=&_manager->ntuple("NTUPLE",1);
  _ntuple->setColumnWise();
  _ntuple->block("var", chform_var, var_addr);

  cout << name( ) << " begin Job" << endl;
  return AppResult::OK;
}

AppResult HepgHist::beginRun( AbsEvent* aRun )
{
  cout << name( ) << " begin Run" << endl;
  return AppResult::OK;
}

AppResult HepgHist::event( AbsEvent* anEvent )
{
  cout << name( ) << " event entry" << endl;

// For CDF users, this is an example of how to access event data in a user
// written module.  Note that a pointer to the event is passed to the event
// entry, and that entry returns the (optionally) modified event.
//
  EventRecord::ConstIterator iHEPG( anEvent, "HEPG_StorableBank") ;
  if (iHEPG.is_valid()) {
    ConstHandle<HEPG_StorableBank> h_HEPG(iHEPG) ;

    cout << name() << ": dais_version_number: " <<
      h_HEPG->dais_version_number() << endl;
    cout << name() << ": event_number: " <<
      h_HEPG->event_number() << endl;
    cout << name() << ": n_particles: " <<
      h_HEPG->n_particles() << endl;
    cout << name() << ": first_displaced: " <<
      h_HEPG->first_displaced() << endl;
    cout << name() << ": n_displaced: " <<
      h_HEPG->n_displaced() << endl;
    cout << name() << ": n_primary: " <<
      h_HEPG->n_primary() << endl;

    // iterate over all particle in HEPG bank
    for(HEPG_StorableBank::particleIter partIter(*h_HEPG);
	partIter.is_valid(); ++partIter) {
      int idx= partIter.index();
      /*
      cout << partIter.index() << ' ' << partIter.idhep() << ' '
	   << partIter.istdhep() << ' '
	   << partIter.jmo1hep() << ' ' << partIter.jmo2hep() << ' '
	   << partIter.jda1hep() << ' ' << partIter.jda2hep() << ' '
	   << endl;
	   */
      // tau 15
      if (h_HEPG->idhep(idx) == 15 || h_HEPG->idhep(idx) == -15 ) {
	_var.idhep=h_HEPG->idhep(idx);
	_var.E=h_HEPG->E(idx);
	_var.Px=h_HEPG->Px(idx);
	_var.Py=h_HEPG->Py(idx);
	_var.Pz=h_HEPG->Pz(idx);
	_var.Vx=h_HEPG->Vx(idx);
	_var.Vy=h_HEPG->Vy(idx);
	_var.Vz=h_HEPG->Vz(idx);
	_ntuple->captureBlock("var");
	_ntuple->storeCapturedData();
	_ntuple->clearDataBlock("var");
      }
    }
  }
  return AppResult::OK;
}

AppResult HepgHist::other( AbsEvent* )
{
  cout << name( ) << " other" << endl;
  return AppResult::OK;
}

AppResult HepgHist::endRun( AbsEvent* aRun )
{
  cout << name( ) << " end Run" << endl;
  return AppResult::OK;
}

AppResult HepgHist::endJob( AbsEvent* aJob )
{
  cout << name( ) << " end Job" << endl;
  return AppResult::OK;
}

AppResult HepgHist::abortJob( AbsEvent* aJob )
{
  cout << name( ) << " abort Job" << endl;
  return AppResult::OK;
}

AppModule* HepgHist::clone(const char* cloneName)
{
  return new HepgHist(cloneName,"this module is a clone HepgHist");
}

const char* HepgHist::rcsId( ) const
{
  return rcsid;
}
